#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include <X11/Xlib.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

#ifdef __cplusplus
extern "C"{
#endif
static void Initialize();
static void DrawWin1(GLenum);
static void DrawBack(GLenum);
static void DrawCube(GLenum);
static void SetPickMatrix(GLdouble,GLdouble,GLdouble,GLdouble,GLint*);
static void DumpHits(GLint,GLuint*);
static Window CreateWindow(const char*,int,int,unsigned int,unsigned int);
static void GetWindowSize(Display*,Window,int*,int*);
static Bool WaitForNotify(Display*,XEvent*,char*);
#ifdef __cplusplus
}
#endif

static GLubyte plus_5_5[] = {
  0x20,
  0x20,
  0xf8,
  0x20,
  0x20,
};

static struct 
{
  Display*      display;
  Colormap      colormap;
  XVisualInfo*  vinfo;
  GLXContext    ctx;
  int           privateColormap;
} Class = {NULL,0L,NULL,NULL,1};

int main(int a_argn,char*  a_args[]){
  Window win1,win2,win3;
  int    width,height;
  GLint  viewport[4];
  double aspect;
  GLuint list;
  /*...............*/
  if(a_argn==2) {
    Class.privateColormap = 0;
  }

  Initialize();

  /*Set window 1.*/
  win1 = CreateWindow("win 1",0,0,400,200);
  if(glXMakeCurrent(Class.display,win1,Class.ctx)==False){
    printf("glXMakeCurrent failed.\n");
  }
  GetWindowSize(Class.display,win1,&width,&height);

/* Produce a white line on a black window
  glBegin     (GL_LINES);
  glVertex3d  (-0.9,-0.9,0.);
  glVertex3d  ( 0.9,0.9,0.);
  glEnd       ();
  glFinish    ();
  MainLoop    ();
*/
    glEnable(GL_LINE_STIPPLE);
  glEnable       (GL_DEPTH_TEST);
  glEnable       (GL_SCISSOR_TEST);
  glShadeModel   (GL_FLAT);
  glViewport     (0,0,width,height);
  glScissor      (0,0,width,height);
  glClearColor   (0.8,0.8,0.8,0.);
/*glClear        (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);*/ /*C++ do not compile this !.*/
  glClear        (GL_COLOR_BUFFER_BIT);
  glClear        (GL_DEPTH_BUFFER_BIT);

  glViewport     (width/4,height/2,width/2,height/2);
  glScissor      (width/4,height/2,width/2,height/2);

  glGetIntegerv  (GL_VIEWPORT,viewport);
  aspect         = ((double)viewport[2])/((double)viewport[3]);

  glMatrixMode   (GL_PROJECTION); 
  glLoadIdentity ();
/*glFrustum      (-aspect,aspect,-1.,1.,1.,4.);*/
  glOrtho        (-aspect,aspect,-1.,1.,-10.,10.);
  glMatrixMode   (GL_MODELVIEW);

  glClearColor   (0.,0.,0.,0.);
  glClear        (GL_COLOR_BUFFER_BIT);
  glClear        (GL_DEPTH_BUFFER_BIT);
  DrawWin1       (GL_RENDER);

  glViewport     (0,0,width,height/4);
  glScissor      (0,0,width,height/4);
  glGetIntegerv  (GL_VIEWPORT,viewport);
  aspect         = ((double)viewport[2])/((double)viewport[3]);

  glMatrixMode   (GL_PROJECTION); 
  glLoadIdentity ();
  glOrtho        (0.,aspect * 1.1,-0.1,1.,-10.,10.);

  glClearColor   (0.4,0.4,0.4,0.);
  glClear        (GL_COLOR_BUFFER_BIT);
  glClear        (GL_DEPTH_BUFFER_BIT);

  glMatrixMode   (GL_MODELVIEW);
  glLoadIdentity ();
  glColor3d      (1.0,1.0,1.0);   
  glScaled       (0.3,1.,1.);

  glFinish       ();
  glXSwapBuffers (Class.display,win1);

/*Set window 3.*/
  win3           = CreateWindow("win 3",0    ,450,200,400);
  if(glXMakeCurrent(Class.display,win3,Class.ctx)==False)
    {
      printf("glXMakeCurrent failed.\n");
    }
  GetWindowSize  (Class.display,win3,&width,&height);

  glEnable       (GL_DEPTH_TEST);
  glDisable      (GL_SCISSOR_TEST);
  glShadeModel   (GL_FLAT);
  glViewport     (0,0,width,height);
  glGetIntegerv  (GL_VIEWPORT,viewport);
  aspect         = ((double)viewport[2])/((double)viewport[3]);
  glMatrixMode   (GL_PROJECTION); 
  glLoadIdentity ();
  glFrustum      (-aspect,aspect,-1.,1.,1.,4.);
  glMatrixMode   (GL_MODELVIEW);

  glLoadIdentity ();
  gluLookAt      (3.,0.,0.,  /*eye*/
                  0.,0.,0.,  /*center*/
                  0.,-1.,0.); /*up*/

  glClearColor   (0.3,0.3,0.3,0.);
  glClear        (GL_COLOR_BUFFER_BIT);
  glClear        (GL_DEPTH_BUFFER_BIT);
  DrawBack       (GL_RENDER);
  DrawCube       (GL_RENDER);
  glFinish       ();
  glXSwapBuffers (Class.display,win3);

/*Set window 2.*/
  win2           = CreateWindow("win 2",450  ,  0,200,400);
  if(glXMakeCurrent(Class.display,win2,Class.ctx)==False)
    {
      printf("glXMakeCurrent failed.\n");
    }
  GetWindowSize  (Class.display,win2,&width,&height);

  glEnable       (GL_DEPTH_TEST);
  glDisable      (GL_SCISSOR_TEST);
  glShadeModel   (GL_FLAT);
  glViewport     (0,0,width,height);
  glGetIntegerv  (GL_VIEWPORT,viewport);
  aspect         = ((double)viewport[2])/((double)viewport[3]);
  glMatrixMode   (GL_PROJECTION); 
  glLoadIdentity ();
  glFrustum      (-aspect,aspect,-1.,1.,1.,4.);
  glMatrixMode   (GL_MODELVIEW);

  glClearColor   (0.5,0.5,0.5,0.0);

  list           = glGenLists (1);
  glNewList      (list,GL_COMPILE);
  DrawCube       (GL_RENDER);
  glEndList      ();

  {int count;for(count=0;count<20;count++) {
  glClear        (GL_COLOR_BUFFER_BIT);
  glClear        (GL_DEPTH_BUFFER_BIT);
  glLoadIdentity ();
  glTranslated   (0.,0.,-2.);
  DrawBack       (GL_RENDER);
  glRotated      (5. * count,0.,1.,0.);
  /*  DrawCube       (GL_RENDER);*/
  glCallList     (list);
  glFinish       ();
  glXSwapBuffers (Class.display,win2);
  glXWaitX       ();
  }}

  glDeleteLists  (list,1);

  glXWaitX       ();

  while(1) { 
    XEvent      xevent;
    if(XPending(Class.display)!=0) {
      XNextEvent(Class.display,&xevent);
      if(xevent.type==ButtonPress && xevent.xbutton.button==1) {
#define BUFSIZE 512
        GLuint         selectBuffer[BUFSIZE];
        GLint          hitn;
        double         xpick,ypick;
        /*Picking on window 1.*/
        /*Set window 1.*/
        if(glXMakeCurrent (Class.display,win1,Class.ctx)==False) {
          printf("glXMakeCurrent failed.\n");
          exit(EXIT_FAILURE);
	}
        GetWindowSize (Class.display,win1,&width,&height);

        glSelectBuffer (BUFSIZE,selectBuffer);
        glRenderMode   (GL_SELECT);
        glInitNames    ();
        glPushName     (0);

        glShadeModel   (GL_FLAT);
        glViewport     (0,0,width,height);
        glClearColor   (0.8,0.8,0.8,0.);
        glClear        (GL_COLOR_BUFFER_BIT);
        glClear        (GL_DEPTH_BUFFER_BIT);
	      
        glViewport     (width/4,height/2,width/2,height/2);

        glGetIntegerv  (GL_VIEWPORT,viewport);
        xpick          = (GLdouble)xevent.xbutton.x;
        ypick          = (GLdouble)height - (GLdouble)xevent.xbutton.y; 

        glMatrixMode   (GL_PROJECTION); 
        glLoadIdentity ();
        SetPickMatrix  (xpick,ypick,6.,6.,viewport);
        glMatrixMode   (GL_MODELVIEW);

        glClearColor   (0.,0.,0.,0.);
        glClear        (GL_COLOR_BUFFER_BIT);
        glClear        (GL_DEPTH_BUFFER_BIT);
        DrawWin1       (GL_SELECT);

        hitn           = glRenderMode   (GL_RENDER);
        DumpHits       (hitn,selectBuffer);

      } else if(xevent.type==ButtonPress && xevent.xbutton.button==2) {
        glXDestroyContext (Class.display,Class.ctx);
        a_args = NULL;
        exit(EXIT_SUCCESS);
      }
    }
  }
}

static void SetPickMatrix(GLdouble a_x,GLdouble a_y,GLdouble a_width,GLdouble a_height,GLint* a_viewport){
  double l,r,b,t;
  double aspect,x,y;
  double ln,rn,bn,tn;
  /*..........................................................*/
  aspect = ((GLdouble)a_viewport[2])/((GLdouble)a_viewport[3]);
 
  l = - aspect;
  r =   aspect;
  b = - 1.;
  t =   1.;
 
  x  = a_x - ((GLdouble)a_viewport[0]);
  y  = a_y - ((GLdouble)a_viewport[1]);

  ln = l + (x -  a_width/2.) * (r-l) / ((GLdouble)a_viewport[2]);
  rn = l + (x +  a_width/2.) * (r-l) / ((GLdouble)a_viewport[2]);
  bn = b + (y - a_height/2.) * (t-b) / ((GLdouble)a_viewport[3]);
  tn = b + (y + a_height/2.) * (t-b) / ((GLdouble)a_viewport[3]);

  glViewport     ((GLint)(a_x - a_width/2.),(GLint)(a_y - a_height/2.),(GLsizei)a_width,(GLsizei)a_height);
/*glFrustum      (ln,rn,bn,tn,1.,4.);*/
  glOrtho        (ln,rn,bn,tn,-10.,10.);
}

static void DumpHits(GLint a_hitn,GLuint* a_selectBuffer){
  int count;
  GLuint* ptr;
  /*...................*/
  ptr = a_selectBuffer; 
  for(count=0;count<a_hitn;count++) { 
    GLuint namen,namei;
    printf("hit:\n");
    namen = *ptr;ptr++;
    printf("  number of names:%d\n",namen);
    printf("               z1:%d\n",*ptr);ptr++;
    printf("               z2:%d\n",*ptr);ptr++;
    for(namei=0;namei<namen;namei++) {
      printf("            namei:%d\n",*ptr);ptr++;
    }
  }
}

static void DrawWin1(GLenum a_mode){
  glLoadIdentity ();
  glRotated      (45.,1.,1.,0.);
  if(a_mode==GL_SELECT) glLoadName (100);
  DrawBack       (GL_RENDER);
  DrawCube       (a_mode);
  glLoadIdentity ();
  glTranslated   (1.5,0.,0.);
  glRotated      (45.,1.,1.,0.);
/*glScaled       (0.5,0.5,0.5);*/
  if(a_mode==GL_SELECT) glLoadName (200);
  DrawBack       (GL_RENDER);
  DrawCube       (a_mode);
}
static void DrawBack(GLenum a_mode){
  if(a_mode==GL_SELECT) glPushName (40);

  glColor3d      (0.7,0.7,0.7);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 1.0,-1.0, 1.0);
  glVertex3d     ( 1.0,-1.0,-1.0);
  glVertex3d     (-1.0,-1.0,-1.0);
  glVertex3d     (-1.0,-1.0, 1.0);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();
}

static void DrawCube(GLenum a_mode){
  glLineWidth    (4.0);

  if(a_mode==GL_SELECT) glPushName (10);

  glColor3d      (1.0,0.0,0.);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 0.5,-0.5,0.5);
  glVertex3d     ( 0.5, 0.5,0.5);
  glVertex3d     (-0.5, 0.5,0.5);
  glVertex3d     (-0.5,-0.5,0.5);
  glEnd          ();

  glColor3d      (0.0,0.0,1.0);   
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
  glRasterPos3f(0.5,-0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f( 0.5, 0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f(-0.5, 0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f(-0.5,-0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);

  if(a_mode==GL_SELECT) glPopName  ();

  if(a_mode==GL_SELECT) glPushName (20);

  glColor3d      (0.0,1.0,0.);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 0.5,-0.5,-0.5);
  glVertex3d     ( 0.5, 0.5,-0.5);
  glVertex3d     (-0.5, 0.5,-0.5);
  glVertex3d     (-0.5,-0.5,-0.5);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();

  if(a_mode==GL_SELECT) glPushName (30);

  glColor3d      (1.0,1.0,0.);   
  glBegin        (GL_LINES);
  glVertex3d     ( 0.5,-0.5, 0.5);
  glVertex3d     ( 0.5,-0.5,-0.5);
  glVertex3d     ( 0.5, 0.5, 0.5);
  glVertex3d     ( 0.5, 0.5,-0.5);
  glVertex3d     (-0.5, 0.5, 0.5);
  glVertex3d     (-0.5, 0.5,-0.5);
  glVertex3d     (-0.5,-0.5, 0.5);
  glVertex3d     (-0.5,-0.5,-0.5);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();

}

static void Initialize(){
  static int attributeList[] = { GLX_RGBA,
		    GLX_RED_SIZE, 2,
		    GLX_GREEN_SIZE, 2,
		    GLX_BLUE_SIZE, 2,
		    GLX_DOUBLEBUFFER,
		    GLX_DEPTH_SIZE, 1,
		    None };
  /*...................................*/
  Class.display = XOpenDisplay(NULL);
  if(Class.display==NULL) {
    printf("Can't open display.\n");
    exit(EXIT_SUCCESS);
  }

  Class.vinfo = 
    glXChooseVisual(Class.display,DefaultScreen(Class.display),attributeList);
  if(Class.vinfo==NULL) {
    printf("Can't choose a visual.\n");
    exit(EXIT_SUCCESS);
  }

  Class.ctx = glXCreateContext(Class.display,Class.vinfo,NULL,GL_FALSE);
  if(Class.ctx==NULL) {
    printf("Can't create a GLX context.\n");
    exit(EXIT_SUCCESS);
  }

  if(Class.privateColormap==1) {
    /* It is better to create a colormap adapted to the visual.*/
    Class.colormap = XCreateColormap(Class.display,XDefaultRootWindow(Class.display),Class.vinfo->visual,AllocNone); 
  } else {
    Class.colormap = 
      XDefaultColormap(Class.display,DefaultScreen(Class.display));
  }
  if(Class.colormap==0L) {
    printf("Can't create X colormap.\n");
    exit(EXIT_SUCCESS);
  }
}

static Window CreateWindow(const char* title,int a_x,int a_y,unsigned int a_width,unsigned int a_height) {
  Window window;
  XSetWindowAttributes swa;
  XSizeHints sh;
  XTextProperty tp;
  /*................................*/
  swa.colormap = Class.colormap;
  swa.border_pixel = 0L;
  swa.event_mask = StructureNotifyMask | ExposureMask | ButtonPressMask;
  window           = XCreateWindow (Class.display, 
                                    XDefaultRootWindow(Class.display),
                                    a_x,a_y,a_width,a_height,
				    0,
                                    Class.vinfo->depth,
				    InputOutput,
                                    Class.vinfo->visual,
				    CWBorderPixel|CWColormap|CWEventMask,&swa);
  if(window==0L) {
    printf("Can't create an X window.\n");
    exit(EXIT_SUCCESS);
  }

  XStringListToTextProperty((char**)&title,1,&tp);
  sh.flags = USPosition | USSize;
  XSetWMProperties(Class.display, window, &tp, &tp, 0, 0, &sh, 0, 0);
  XFree(tp.value);

  XMapWindow(Class.display, window);
  XRaiseWindow(Class.display, window);
 {XEvent event;
  XIfEvent(Class.display, &event, WaitForNotify, (char*)window);}

  return window;
}

static void GetWindowSize(Display* a_display,Window a_window,int* a_width,int* a_height){
  XWindowAttributes watbs;
  XGetWindowAttributes(a_display,a_window,&watbs);
  *a_width  = watbs.width;
  *a_height = watbs.height;
}

static Bool WaitForNotify(Display* d,XEvent* e,char* arg){
  d = NULL;
  return (e->type == MapNotify) && (e->xmap.window == (Window)arg);
}
