#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include <GL/gl.h>

#ifdef __cplusplus
extern "C"{
#endif
void SetPickMatrix(GLdouble,GLdouble,GLdouble,GLdouble,GLint*);
void DumpHits(GLint,GLuint*);
void DrawWin1(GLenum);
void DrawBack(GLenum);
void DrawCube(GLenum);
#ifdef __cplusplus
}
#endif

static GLubyte plus_5_5[] = {
  0x20,
  0x20,
  0xf8,
  0x20,
  0x20,
};

void SetPickMatrix(GLdouble a_x,GLdouble a_y,GLdouble a_width,GLdouble a_height,GLint* a_viewport){
  double l,r,b,t;
  double aspect,x,y;
  double ln,rn,bn,tn;
  /*..........................................................*/
  aspect = ((GLdouble)a_viewport[2])/((GLdouble)a_viewport[3]);
 
  l = - aspect;
  r =   aspect;
  b = - 1.;
  t =   1.;
 
  x  = a_x - ((GLdouble)a_viewport[0]);
  y  = a_y - ((GLdouble)a_viewport[1]);

  ln = l + (x -  a_width/2.) * (r-l) / ((GLdouble)a_viewport[2]);
  rn = l + (x +  a_width/2.) * (r-l) / ((GLdouble)a_viewport[2]);
  bn = b + (y - a_height/2.) * (t-b) / ((GLdouble)a_viewport[3]);
  tn = b + (y + a_height/2.) * (t-b) / ((GLdouble)a_viewport[3]);

  glViewport     ((GLint)(a_x - a_width/2.),(GLint)(a_y - a_height/2.),(GLsizei)a_width,(GLsizei)a_height);
/*glFrustum      (ln,rn,bn,tn,1.,4.);*/
  glOrtho        (ln,rn,bn,tn,-10.,10.);
}

void DumpHits(GLint a_hitn,GLuint* a_selectBuffer){
  int count;
  GLuint* ptr;
  /*...................*/
  ptr = a_selectBuffer; 
  for(count=0;count<a_hitn;count++) { 
    GLuint namen,namei;
    printf("hit:\n");
    namen = *ptr;ptr++;
    printf("  number of names:%d\n",namen);
    printf("               z1:%d\n",*ptr);ptr++;
    printf("               z2:%d\n",*ptr);ptr++;
    for(namei=0;namei<namen;namei++) {
      printf("            namei:%d\n",*ptr);ptr++;
    }
  }
}

void DrawWin1(GLenum a_mode){
  glLoadIdentity ();
  glRotated      (45.,1.,1.,0.);
  if(a_mode==GL_SELECT) glLoadName (100);
  DrawBack       (GL_RENDER);
  DrawCube       (a_mode);
  glLoadIdentity ();
  glTranslated   (1.5,0.,0.);
  glRotated      (45.,1.,1.,0.);
/*glScaled       (0.5,0.5,0.5);*/
  if(a_mode==GL_SELECT) glLoadName (200);
  DrawBack       (GL_RENDER);
  DrawCube       (a_mode);
}
void DrawBack(GLenum a_mode){
  if(a_mode==GL_SELECT) glPushName (40);

  glColor3d      (0.7,0.7,0.7);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 1.0,-1.0, 1.0);
  glVertex3d     ( 1.0,-1.0,-1.0);
  glVertex3d     (-1.0,-1.0,-1.0);
  glVertex3d     (-1.0,-1.0, 1.0);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();
}

void DrawCube(GLenum a_mode){
  glLineWidth    (4.0);

  if(a_mode==GL_SELECT) glPushName (10);

  glColor3d      (1.0,0.0,0.);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 0.5,-0.5,0.5);
  glVertex3d     ( 0.5, 0.5,0.5);
  glVertex3d     (-0.5, 0.5,0.5);
  glVertex3d     (-0.5,-0.5,0.5);
  glEnd          ();

  glColor3d      (0.0,0.0,1.0);   
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
  glRasterPos3f(0.5,-0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f( 0.5, 0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f(-0.5, 0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);
  glRasterPos3f(-0.5,-0.5,0.5);
  glBitmap(5,5,2.,2.,0.,0.,plus_5_5);

  if(a_mode==GL_SELECT) glPopName  ();

  if(a_mode==GL_SELECT) glPushName (20);

  glColor3d      (0.0,1.0,0.);   
  glBegin        (GL_POLYGON);
  glVertex3d     ( 0.5,-0.5,-0.5);
  glVertex3d     ( 0.5, 0.5,-0.5);
  glVertex3d     (-0.5, 0.5,-0.5);
  glVertex3d     (-0.5,-0.5,-0.5);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();

  if(a_mode==GL_SELECT) glPushName (30);

  glColor3d      (1.0,1.0,0.);   
  glBegin        (GL_LINES);
  glVertex3d     ( 0.5,-0.5, 0.5);
  glVertex3d     ( 0.5,-0.5,-0.5);
  glVertex3d     ( 0.5, 0.5, 0.5);
  glVertex3d     ( 0.5, 0.5,-0.5);
  glVertex3d     (-0.5, 0.5, 0.5);
  glVertex3d     (-0.5, 0.5,-0.5);
  glVertex3d     (-0.5,-0.5, 0.5);
  glVertex3d     (-0.5,-0.5,-0.5);
  glEnd          ();

  if(a_mode==GL_SELECT) glPopName  ();

}
