
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>

#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoCube.h>

#include <Inventor/actions/SoBoxHighlightRenderAction.h>

#include <cstdlib>
#include <iostream>

#if defined(__APPLE__)
static void LookDSM_Problem();
#endif

inline void selectionCB(void* aTag,SoPath* aPath)  {
  SoSelection* sel = (SoSelection*)aTag;
  sel->touch(); // to redraw
}

int main(int /*argc*/,char** argv) {
#if defined(__APPLE__)
  LookDSM_Problem();
#endif

  Widget top = SoXt::init(argv[0]);
  if (top==NULL) exit(1);

  // Create a scene :

  SoSelection* root = new SoSelection;
  root->addSelectionCallback(selectionCB,root);
  root->ref();

  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue (SbColor(1,1,0));
  root->addChild(material);

  SoScale* scale = new SoScale;
  scale->scaleFactor.setValue(0.1F,0.1F,0.1F);
  //scale->scaleFactor.setValue(0.5F,0.5F,0.5F);
  root->addChild(scale);

  SoCube* cube = new SoCube;
  root->addChild(cube);

  // Create a 3D viewer :

  SoXtExaminerViewer* viewer = new SoXtExaminerViewer(top);
  viewer->setBackgroundColor(SbColor(0.4F,0.4F,0.4F));
  viewer->setSceneGraph(root);
  viewer->viewAll();
  viewer->setSize(SbVec2s(700,500));
  viewer->setTitle("CoinXt");
  //viewer->setViewing(FALSE);
  //viewer->setDecoration(FALSE);
  //viewer->setAutoClipping(FALSE);

  SoBoxHighlightRenderAction ra;
  viewer->setGLRenderAction(&ra);

  viewer->show();

  SoXt::show(top);

  std::cout << "Type Q to quit." << std::endl;

  SoXt::mainLoop();

  //Cleanup
  root->unref();

  delete viewer;

  return 0;
}

#if defined(__APPLE__) && (XmVERSION>=2) && (XmREVISION>=3)
#include <Xm/Display.h>
#include <Xm/DialogS.h>
#include <Xm/DragOverS.h>
#include <Xm/GrabShell.h>
#include <Xm/Print.h>
#include <X11/IntrinsicP.h>
#include <iostream>
static void ResetVendorField(WidgetClass aWidgetClass){
  WidgetClass wc = aWidgetClass;
  while(wc) {
    WidgetClass super = wc->core_class.superclass;
    if(super) {
      char* name = (char*)super->core_class.class_name;
      if( !strcmp(name,"VendorShell") && (super!=vendorShellWidgetClass) ) {
        // The Vendor super class is not the Motif one, override :
        wc->core_class.superclass = vendorShellWidgetClass;
        return;
      }
    }
    wc = super;
  }
}
static void LookDSM_Problem(){
  if(!vendorShellWidgetClass->core_class.class_part_initialize) {
    // The vendorShellWidgetClass we get is the Xt one.
    std::cout << "OnX::XtUI::LookDSM_Problem :" << std::endl;
    std::cout << " Can't solve the dsm problem." << std::endl;
    std::cout << " Bad linking order of -lXm and -lXt " << std::endl;
    std::cout << " or you try to load a DLL linked to X11 prior to" << std::endl;
    std::cout << " load a DLL linked with Motif." << std::endl;
    return;
  }
  ResetVendorField(xmDisplayClass);
  ResetVendorField(xmDialogShellWidgetClass);
  ResetVendorField(xmDragOverShellWidgetClass);
  ResetVendorField(xmGrabShellWidgetClass);
  ResetVendorField(xmPrintShellWidgetClass);
}
#else
static void LookDSM_Problem(){}
#endif
