
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtPlaneViewer.h>

#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>

#include <cstdlib>
#include <iostream>

#if defined(__APPLE__)
static void LookDSM_Problem();
#endif

int main(int /*argc*/,char** argv) {
#if defined(__APPLE__)
  LookDSM_Problem();
#endif

  Widget top = SoXt::init(argv[0]);
  if (top==NULL) exit(1);

  // Create a scene :

  SoSeparator* root = new SoSeparator;
  root->ref();

  SoOrthographicCamera* camera = new SoOrthographicCamera;
  camera->height.setValue(4);
  camera->position.setValue(SbVec3f(0,0,1));
  root->addChild(camera);

  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue (SbColor(1,1,0));
  root->addChild(material);

  SbVec3f points[5];
  int32_t pointn = 5;
  float size = 1.5F;
  points[0].setValue(-size,-size,0);
  points[1].setValue( size,-size,0);
  points[2].setValue( size, size,0);
  points[3].setValue(-size, size,0);
  points[4].setValue(-size,-size,0);
  SoCoordinate3* coordinate3 = new SoCoordinate3;
  coordinate3->point.setValues(0,pointn,points);
  root->addChild(coordinate3);

  SoLineSet* lineSet = new SoLineSet;
  lineSet->numVertices.setValues(0,1,&pointn);
  root->addChild(lineSet);

  // Create a 2D viewer :

  SoXtPlaneViewer* viewer = new SoXtPlaneViewer(top);
  viewer->setBackgroundColor(SbColor(0.4F,0.4F,0.4F));
  viewer->setSceneGraph(root);
  viewer->setCamera(camera);
  viewer->setSize(SbVec2s(700,500));
  viewer->setTitle("CoinXt_2D");
  //viewer->setViewing(FALSE);
  //viewer->setDecoration(FALSE);
  viewer->show();

  SoXt::show(top);

  std::cout << "Type Q to quit." << std::endl;

  SoXt::mainLoop();

  //Cleanup
  root->unref();

  delete viewer;

  return 0;
}

#if defined(__APPLE__) && (XmVERSION>=2) && (XmREVISION>=3)
#include <Xm/Display.h>
#include <Xm/DialogS.h>
#include <Xm/DragOverS.h>
#include <Xm/GrabShell.h>
#include <Xm/Print.h>
#include <X11/IntrinsicP.h>
#include <iostream>
static void ResetVendorField(WidgetClass aWidgetClass){
  WidgetClass wc = aWidgetClass;
  while(wc) {
    WidgetClass super = wc->core_class.superclass;
    if(super) {
      char* name = (char*)super->core_class.class_name;
      if( !strcmp(name,"VendorShell") && (super!=vendorShellWidgetClass) ) {
        // The Vendor super class is not the Motif one, override :
        wc->core_class.superclass = vendorShellWidgetClass;
        return;
      }
    }
    wc = super;
  }
}
static void LookDSM_Problem(){
  if(!vendorShellWidgetClass->core_class.class_part_initialize) {
    // The vendorShellWidgetClass we get is the Xt one.
    std::cout << "OnX::XtUI::LookDSM_Problem :" << std::endl;
    std::cout << " Can't solve the dsm problem." << std::endl;
    std::cout << " Bad linking order of -lXm and -lXt " << std::endl;
    std::cout << " or you try to load a DLL linked to X11 prior to" << std::endl;
    std::cout << " load a DLL linked with Motif." << std::endl;
    return;
  }
  ResetVendorField(xmDisplayClass);
  ResetVendorField(xmDialogShellWidgetClass);
  ResetVendorField(xmDragOverShellWidgetClass);
  ResetVendorField(xmGrabShellWidgetClass);
  ResetVendorField(xmPrintShellWidgetClass);
}
#else
static void LookDSM_Problem(){}
#endif
